package com.ruoyi.framework.security.filter;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import com.ruoyi.common.constant.CacheConstants;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.domain.vo.UserVO;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.enums.UserStatus;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.web.service.SysPermissionService;
import com.ruoyi.framework.web.service.TokenService;
import com.ruoyi.system.user.service.UserService;

/**
 * token过滤器 验证token有效性
 *
 * @author ruoyi
 */
@Component
public class JwtAuthenticationTokenFilter extends OncePerRequestFilter
{
    @Autowired
    private TokenService tokenService;

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private UserService userService;

    @Autowired
    private SysPermissionService permissionService;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain chain)
            throws ServletException, IOException
    {
        if (StringUtils.isNull(SecurityUtils.getAuthentication())) {

            // 优先尝试使用token来获取用户信息
            LoginUser loginUser = tokenService.getLoginUser(request);
            if (loginUser != null) {
                tokenService.verifyToken(loginUser);
            }
            // 若token未获取到信息，则使用accessKey来获取用户信息
            else {
                loginUser = this.getLoginUserByAccessKey(request);
            }

            if (StringUtils.isNotNull(loginUser)) {
                UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(loginUser, null, loginUser.getAuthorities());
                authenticationToken.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
                SecurityContextHolder.getContext().setAuthentication(authenticationToken);
            }
        }
        chain.doFilter(request, response);
    }


    private LoginUser getLoginUserByAccessKey(HttpServletRequest request) {
        String userId = request.getHeader(Constants.HEADER_USER_ID);
        String apiSecret = request.getHeader(Constants.HEADER_API_SECRET);
        if (StringUtils.isBlank(userId) || StringUtils.isBlank(apiSecret)) {
            return null;
        }
        // 从缓存中获取数据
        String redisKey = CacheConstants.API_USER_ID_PREFIX + userId;
        LoginUser loginUser = redisCache.getCacheObject(redisKey);

        // 缓存未命中，查询获取数据
        if (loginUser == null) {

            // 获取用户信息
            UserVO user = userService.selectUserById(Long.parseLong(userId));
            if (user == null || !UserStatus.isEnabled(user.getStatus())) {
                return null;
            }
            loginUser = new LoginUser(user.getUserId(), user.getDeptId(), user, permissionService.getMenuPermission(user));

            // 放入缓存中
            redisCache.setCacheObject(redisKey, loginUser, 30, TimeUnit.MINUTES);
        }

        // 校验秘钥
        if (!SecurityUtils.matchesPassword(apiSecret, loginUser.getUser().getApiSecret())) {
            redisCache.deleteObject(redisKey);
            return null;
        }

        return loginUser;
    }
}
